package org.bouncycastle.jce.cert;

import java.security.InvalidAlgorithmParameterException;
import java.util.Collection;

/**
 * The <i>Service Provider Interface</i> (<b>SPI</b>)
 * for the {@link CertStore CertStore} class. All  CertStore</code>
 * implementations must include a class (the SPI class) that extends
 * this class ( CertStoreSpi</code>), provides a constructor with
 * a single argument of type  CertStoreParameters</code>, and implements 
 * all of its methods. In general, instances of this class should only be
 * accessed through the  CertStore</code> class. 
 * For details, see the Java Cryptography Architecture.<br />
 * <br />
 * <b>Concurrent Access</b><br />
 * <br />
 * The public methods of all  CertStoreSpi</code> objects must be 
 * thread-safe. That is, multiple threads may concurrently invoke these 
 * methods on a single  CertStoreSpi</code> object (or more than one) 
 * with no ill effects. This allows a  CertPathBuilder</code> to search 
 * for a CRL while simultaneously searching for further certificates, for 
 * instance.<br />
 * <br />
 * Simple  CertStoreSpi</code> implementations will probably ensure
 * thread safety by adding a  synchronized</code> keyword to their
 *  engineGetCertificates</code> and  engineGetCRLs</code> methods.
 * More sophisticated ones may allow truly concurrent access.
 **/
public abstract class CertStoreSpi
    extends Object
{

    /**
     * The sole constructor.
     *
     * @param params the initialization parameters (may be  null</code>)
     * @exception InvalidAlgorithmParameterException if the initialization
     * parameters are inappropriate for this  CertStoreSpi</code>
     */
    public CertStoreSpi(CertStoreParameters params)
    throws InvalidAlgorithmParameterException {}

    /**
     * Returns a  Collection</code> of  Certificate</code>s that
     * match the specified selector. If no  Certificate</code>s
     * match the selector, an empty  Collection</code> will be returned.<br />
     * <br />
     * For some  CertStore</code> types, the resulting
     *  Collection</code> may not contain <b>all</b> of the
     *  Certificate</code>s that match the selector. For instance,
     * an LDAP  CertStore</code> may not search all entries in the
     * directory. Instead, it may just search entries that are likely to
     * contain the  Certificate</code>s it is looking for.<br />
     * <br />
     * Some  CertStore</code> implementations (especially LDAP
     *  CertStore</code>s) may throw a  CertStoreException</code>
     * unless a non-null  CertSelector</code> is provided that includes
     * specific criteria that can be used to find the certificates. Issuer
     * and/or subject names are especially useful criteria.
     *
     * @param selector A  CertSelector</code> used to select which
     *   Certificate</code>s should be returned. Specify  null</code>
     *  to return all  Certificate</code>s (if supported).
     *
     * @return A  Collection</code> of  Certificate</code>s that
     *         match the specified selector (never  null</code>)
     *
     * @exception CertStoreException if an exception occurs 
     */
    public abstract Collection engineGetCertificates(CertSelector selector)
    throws CertStoreException;

    /**
     * Returns a  Collection</code> of  CRL</code>s that
     * match the specified selector. If no  CRL</code>s
     * match the selector, an empty  Collection</code> will be returned.<br />
     * <br />
     * For some  CertStore</code> types, the resulting
     *  Collection</code> may not contain <b>all</b> of the
     *  CRL</code>s that match the selector. For instance,
     * an LDAP  CertStore</code> may not search all entries in the
     * directory. Instead, it may just search entries that are likely to
     * contain the  CRL</code>s it is looking for. <br />
     * <br />
     * Some  CertStore</code> implementations (especially LDAP 
     *  CertStore</code>s) may throw a  CertStoreException</code> 
     * unless a non-null  CRLSelector</code> is provided that includes 
     * specific criteria that can be used to find the CRLs. Issuer names 
     * and/or the certificate to be checked are especially useful.
     *
     * @param selector A  CRLSelector</code> used to select which
     *   CRL</code>s should be returned. Specify  null</code>
     *  to return all  CRL</code>s (if supported).
     *
     * @return A  Collection</code> of  CRL</code>s that
     *         match the specified selector (never  null</code>)
     *
     * @exception CertStoreException if an exception occurs 
     */
    public abstract Collection engineGetCRLs(CRLSelector selector)
    throws CertStoreException;
}

